==> ocaml-doc_3.04-4.tar.gz: ocaml-doc-3.04/examples/minicaml/interp.ml <==
open Syntaxe;;
open Eval;;

let code_nombre n =
    Val_nombre n
and dcode_nombre = function
  | Val_nombre n -> n
  | _ -> raise(Erreur "entier attendu")
and code_boolen b =
    Val_boolenne b
and dcode_boolen = function
  | Val_boolenne b -> b
  | _ -> raise(Erreur "boolen attendu");;

(* Pour transformer une fonction Caml en valeur fonctionnelle *)

let prim1 codeur calcul dcodeur =
  Val_primitive (function v -> codeur (calcul (dcodeur v)))
and prim2 codeur calcul dcodeur1 dcodeur2 =
  Val_primitive (function
   | Val_paire (v1, v2) ->
       codeur (calcul (dcodeur1 v1) (dcodeur2 v2))
   | _ -> raise (Erreur "paire attendue"));;

(* L'environnement initial *)

let env_initial =
  ["+",  prim2 code_nombre  ( + ) dcode_nombre dcode_nombre;
   "-",  prim2 code_nombre  ( - ) dcode_nombre dcode_nombre;
   "*",  prim2 code_nombre  ( * ) dcode_nombre dcode_nombre;
   "/",  prim2 code_nombre  ( / ) dcode_nombre dcode_nombre;
   "=",  prim2 code_boolen ( = ) dcode_nombre dcode_nombre;
   "<>", prim2 code_boolen ( <> ) dcode_nombre dcode_nombre;
   "<",  prim2 code_boolen ( < ) dcode_nombre dcode_nombre;
   ">",  prim2 code_boolen ( > ) dcode_nombre dcode_nombre;
   "<=", prim2 code_boolen ( <= ) dcode_nombre dcode_nombre;
   ">=", prim2 code_boolen ( >= ) dcode_nombre dcode_nombre;
   "not", prim1 code_boolen ( not ) dcode_boolen;
   "read_int", prim1 code_nombre (fun x -> read_int ()) dcode_nombre;
   "write_int", prim1 code_nombre
                      (fun x -> print_int x; print_newline (); 0)
                      dcode_nombre];;
let boucle () =
  let env_global = ref env_initial in
  let flux_d'entre = Stream.of_channel stdin in
  try
    while true do
      print_string "# "; flush stdout;
      try
        match lire_phrase flux_d'entre with
        | Expression expr ->
            let rs = value !env_global expr in
            print_string "- = "; imprime_valeur rs;
            print_newline ()
        | Dfinition df ->
            let nouvel_env = value_dfinition !env_global df in
            begin match nouvel_env with
            | (nom, v) :: _ ->
                print_string nom; print_string " = ";
                imprime_valeur v; print_newline ()
            | _ -> failwith "mauvaise gestion des dfinitions"
            end;
            env_global := nouvel_env
      with
      | Stream.Error s ->
          print_string ("Erreur de syntaxe: " ^ s); print_newline ()
      | Erreur msg ->
          print_string "Erreur  l'valuation: "; print_string msg;
          print_newline ()
    done
  with Stream.Failure -> ();;

if not !Sys.interactive then boucle ();;

==> ocaml-doc_3.08.0.orig.tar.gz: ocaml-doc-3.08.0/examples/minicaml/interp.ml <==
(***********************************************************************)
(*                                                                     *)
(*                           Objective Caml                            *)
(*                                                                     *)
(*               Pierre Weis, projet Cristal, INRIA Rocquencourt       *)
(*                                                                     *)
(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
(*  en Automatique.  All rights reserved.  This file is distributed    *)
(*  only by permission.                                                *)
(*                                                                     *)
(***********************************************************************)
open Syntaxe;;
open Eval;;

let code_nombre n =
    Val_nombre n
and dcode_nombre = function
  | Val_nombre n -> n
  | _ -> raise(Erreur "entier attendu")
and code_boolen b =
    Val_boolenne b
and dcode_boolen = function
  | Val_boolenne b -> b
  | _ -> raise(Erreur "boolen attendu");;

(* Pour transformer une fonction Caml en valeur fonctionnelle *)

let prim1 codeur calcul dcodeur =
  Val_primitive (function v -> codeur (calcul (dcodeur v)))
and prim2 codeur calcul dcodeur1 dcodeur2 =
  Val_primitive (function
   | Val_paire (v1, v2) ->
       codeur (calcul (dcodeur1 v1) (dcodeur2 v2))
   | _ -> raise (Erreur "paire attendue"));;

(* L'environnement initial *)

let env_initial =
  ["+",  prim2 code_nombre  ( + ) dcode_nombre dcode_nombre;
   "-",  prim2 code_nombre  ( - ) dcode_nombre dcode_nombre;
   "*",  prim2 code_nombre  ( * ) dcode_nombre dcode_nombre;
   "/",  prim2 code_nombre  ( / ) dcode_nombre dcode_nombre;
   "=",  prim2 code_boolen ( = ) dcode_nombre dcode_nombre;
   "<>", prim2 code_boolen ( <> ) dcode_nombre dcode_nombre;
   "<",  prim2 code_boolen ( < ) dcode_nombre dcode_nombre;
   ">",  prim2 code_boolen ( > ) dcode_nombre dcode_nombre;
   "<=", prim2 code_boolen ( <= ) dcode_nombre dcode_nombre;
   ">=", prim2 code_boolen ( >= ) dcode_nombre dcode_nombre;
   "not", prim1 code_boolen ( not ) dcode_boolen;
   "read_int", prim1 code_nombre (fun x -> read_int ()) dcode_nombre;
   "write_int", prim1 code_nombre
                      (fun x -> print_int x; print_newline (); 0)
                      dcode_nombre];;
let boucle () =
  let env_global = ref env_initial in
  let flux_d'entre = Stream.of_channel stdin in
  try
    while true do
      print_string "# "; flush stdout;
      try
        match lire_phrase flux_d'entre with
        | Expression expr ->
            let rs = value !env_global expr in
            print_string "- = "; imprime_valeur rs;
            print_newline ()
        | Dfinition df ->
            let nouvel_env = value_dfinition !env_global df in
            begin match nouvel_env with
            | (nom, v) :: _ ->
                print_string nom; print_string " = ";
                imprime_valeur v; print_newline ()
            | _ -> failwith "mauvaise gestion des dfinitions"
            end;
            env_global := nouvel_env
      with
      | Stream.Error s ->
          print_string ("Erreur de syntaxe: " ^ s); print_newline ()
      | Erreur msg ->
          print_string "Erreur  l'valuation: "; print_string msg;
          print_newline ()
    done
  with Stream.Failure -> ();;

if not !Sys.interactive then boucle ();;
